#!/bin/sh
#
# $Header: opatch/OPatch/wlskeys/createkeys.sh /st_opatch_11.1/1 2013/07/09 01:49:40 vganesan Exp $
#
# createkeys.sh
#
# Copyright (c) 2013, Oracle and/or its affiliates. All rights reserved. 
#
#    NAME
#      createkeys.sh - <one-line expansion of the name>
#
#    DESCRIPTION
#      <short description of component this file declares/defines>
#
#    NOTES
#      <other useful comments, qualifications, etc.>
#
#    MODIFIED   (MM/DD/YY)
#    billi       06/02/13 - Add check for environment ORACLE_HOME
#    vganesan    05/30/13 - Update some comments
#    billi       03/11/13 - Creation
#

### main ###
binname=`basename $0`
dirpath=`dirname $0`
dirpath=`cd ${dirpath}; pwd`

### parse ###
oh=${ORACLE_HOME}
location=
getoh=0
getloc=0
gethelp=0

for arg in $@
do
    if [ $getoh = 1 ]; then
        oh=$arg
        getoh=2
    fi
    if [ $getloc = 1 ]; then
        location=$arg
        getloc=2
    fi

    if [ "$arg" = "-oh" ]; then
        getoh=1
    fi
    if [ "$arg" = "-location" ]; then
        getloc=1
    fi
    if [ "$arg" = "-help" ]; then
        gethelp=1
    fi
done

### help ###
if [ $gethelp = 1 ] || [ -z "${oh}" -o -z "${location}" ]; then
    echo "Usage:"
    echo "      ${binname} [-help]"
    echo "      ${binname} -oh <OMS Platform home> -location <location to store the config and key files>"
    echo ""
    echo "                 -oh       OMS Platform home path"
    echo "                 -location Location to store the configuration and key files"
    exit 1
fi

case `uname -s` in
Windows_NT*)
  CLASSPATHSEP=\;
;;
CYGWIN*)
  CLASSPATHSEP=\;
;;
*)
  CLASSPATHSEP=:
;;
esac

### check ###
if [ ! -d "$oh" ]; then
    echo "ERROR: Invalid home. Please check and pass OMS platform home through -oh option."
    exit 1;
else
    oh=`cd "$oh"; pwd`
fi
if [ ! -d "$location" ]; then
    echo "ERROR: Invalid location. Please check location input and try again."
    exit 1
else
    location=`cd "$location"; pwd`
fi

MW_HOME=`cd "${oh}/.."; pwd`
SCRIPTPATH=
WLST_HOME=
if [ -f "${MW_HOME}/registry.dat" -a -f "${oh}/bin/emcli" ]; then
    WLST_HOME=${oh}/common/wlst
    SCRIPTPATH=${oh}/common/bin
else
    echo "ERROR: Invalid home. Please check and pass OMS platform home through -oh option."
    exit 1;
fi
export MW_HOME
export WLST_HOME

if [ -x "${SCRIPTPATH}/setHomeDirs.sh" ]; then
    . "${SCRIPTPATH}/setHomeDirs.sh"
fi

# Set the WLST extended env...
if [ -f "${SCRIPTPATH}/setWlstEnv.sh" ] ; then 
  . "${SCRIPTPATH}/setWlstEnv.sh"
fi

# Appending additional jar files to the CLASSPATH...
if [ -d "${WLST_HOME}/lib" ] ; then
    for file in "${WLST_HOME}"/lib/*.jar ; do
       CLASSPATH="${CLASSPATH}${CLASSPATHSEP}${file}"
    done
fi

# Appending additional resource bundles to the CLASSPATH...
if [ -d "${WLST_HOME}/resources" ] ; then
    for file in "${WLST_HOME}"/resources/*.jar ; do
      CLASSPATH="${CLASSPATH}${CLASSPATHSEP}${file}"
   done 
fi

# set up common environment
WLS_NOT_BRIEF_ENV=true
. "${WL_HOME}/server/bin/setWLSEnv.sh"

CLASSPATH="${CLASSPATH}${CLASSPATHSEP}${FMWLAUNCH_CLASSPATH}${CLASSPATHSEP}${DERBY_CLASSPATH}${CLASSPATHSEP}${DERBY_TOOLS}${CLASSPATHSEP}${POINTBASE_CLASSPATH}${CLASSPATHSEP}${POINTBASE_TOOLS}"

if [ "${WLST_HOME}" != "" ] ; then
	WLST_PROPERTIES="-Dweblogic.wlstHome='${WLST_HOME}' ${WLST_PROPERTIES}"
	export WLST_PROPERTIES
fi

export CLASSPATH

echo -n "Please enter weblogic admin server username:"
read username
echo -n "Please enter weblogic admin server password:"
read -s password
echo ""

JVM_ARGS="-Dprod.props.file='${WL_HOME}'/.product.properties ${WLST_PROPERTIES} ${JVM_D64} ${MEM_ARGS} ${CONFIG_JVM_ARGS}"
$JAVA_HOME/bin/java ${JVM_ARGS} -Dweblogic.management.confirmKeyfileCreation=true weblogic.Admin -username $username -password $password -userconfigfile $location/config -userkeyfile $location/key  STOREUSERCONFIG

echo "User configuration file created: $location/config"
echo "User key file created: $location/key"
echo "'createKeys' succeeded."
